unit MainForm;

interface

uses Messages, NSys, NCommand, NConf, NGraphics, NForm, Element, SnapForm;

const
  WM_APP_UPDATE = WM_APP+1;

var
  Command: TCommand;
  Config: TConf;
  ProfDir: string;

type
  TNkMonMainForm = class(TSnapForm)
    AlphaBits: Pointer;
    Elements: array of TElement;
    MemCanvas: TMemCanvas;
    ThreadId: LongWord;
    TerminateThread: Boolean;
    constructor Create; override;
    destructor Destroy; override;
    procedure Closing; override;
    procedure DispatchCommand(Id: Longint); override;
    procedure LoadLayout;
    procedure UpdateElements(var Message: TMessage); message WM_APP_UPDATE;
    procedure MouseDown(const Button: TMouseButton; CX, CY: Longint); override;
    procedure ShowContextMenu(SX, SY: Longint); override;
  published
    procedure cmAbout;
    procedure cmExit;
    procedure cmReloadLayout;
    procedure cmStayOnTop;
  end;

implementation

uses Windows, NLib, Monitor, Menu, Default, AboutDialog;

function ThreadFunc(Parameter: Pointer): Integer;
begin
  with TNkMonMainForm(Parameter) do
    while not TerminateThread do
    begin
      PostMessage(Handle, WM_APP_UPDATE, 0, 0);
      Sleep(1000 - NanoSec div 1000000 mod 1000);
    end;
  Result := 0;
end;

constructor TNkMonMainForm.Create;
begin
  inherited;

  Command := TCommand.Create(Self, [
    cmAbout,
    cmExit,
    cmReloadLayout,
    cmStayOnTop
    ]);

  SetText(App.Name);
  SetWindowLong(Handle, GWL_STYLE, GetWindowLong(Handle, GWL_STYLE)
    and not WS_MAXIMIZEBOX and not WS_MINIMIZEBOX);
  SetWindowLong(Handle, GWL_EXSTYLE,
    GetWindowLong(Handle, GWL_EXSTYLE) or WS_EX_LAYERED);
  SetBorder(False);
  SetToolWindow(True);
  LoadFormPos(Self, Config);

  TimeLocationLatitude := Config.GetFloat('time.location', 0);
  TimeLocationLongitude := Config.GetFloat('time.location', 1);
  TimeLocationAltitude := Config.GetFloat('time.location', 2);

  LoadLayout;
  MemCanvas := TMemCanvas.Create(Handle, 32);
  MemCanvas.BeginDraw;
  SendMessage(Handle, WM_APP_UPDATE, 0, 0);
  BeginThread(nil, 0, ThreadFunc, Self, 0, ThreadId);
  Show;
end;

destructor TNkMonMainForm.Destroy;
begin
  TerminateThread := True;
  Command.Free;
end;

procedure TNkMonMainForm.Closing;
var
  position: TConf;
begin
  position := TConf.Create;
  try
    SaveFormPos(Self, position);
    try
      position.SaveToFile(JoinPath([ProfDir, 'position']));
    except
      on E: EOSError do App.Error(E.Path+#10#10+E.Msg);
    end;
  finally
    position.Free;
  end;
end;

procedure TNkMonMainForm.LoadLayout;
var
  cnt, i: Integer;
  line, path: string;
  s: TStream;
begin
  for i := 0 to High(Elements) do
    Elements[i].Free;
  ClearFontCache;

  Width := 48;
  BgAlpha := 96;
  SetLength(Elements, 10);
  path := JoinPath([ProfDir, 'layout']);
  try
    if IsFile(path) then
      s := TFile.Open(path)
    else
      s := TBytesStream.Create(Layout, True);
  except
    on E: EOSError do
    begin
      Warn(FormatException(E));
      s := TBytesStream.Create(Layout, True);
    end;
  end;
  cnt := 0;
  while TConf.ReadLine(s, line) do
  begin
    if cnt = Length(Elements) then
      SetLength(Elements, cnt + cnt div 4);
    Elements[cnt] := CreateElement(SplitCommandLine(line));
    if Elements[cnt] <> nil then
      Inc(cnt);
  end;
  s.Free;
  SetLength(Elements, cnt);

  Monitor.Init;
  for i := 0 to High(Elements) do
    Elements[i].Init;
end;

procedure TNkMonMainForm.UpdateElements(var Message: TMessage);
var
  i, prevwidth: Integer;
  pt: TPoint;
  size: TSize;
  bf: TBlendFunction;
  c, ec, a: PLongword;
begin
  // alpha: transparent 0<->255 opaque

  prevwidth := Width;
  try
    Monitor.Update;
  except
    on E: EOSError do Warn(SysError(E.ErrNo));
  end;
  for i := 0 to High(Elements) do
    Elements[i].Update;

  // get alpha
  if AlphaBits <> nil then
  begin
    Current.Y := 0;
    MemCanvas.FillRect(0, 0, prevwidth, Height, BgAlpha);
    for i := 0 to High(Elements) do
      Elements[i].DrawAlpha(MemCanvas);
    Move(MemCanvas.Bits^, AlphaBits^, prevwidth*Height*4);
  end;

  // draw
  Current.Y := 0;
  MemCanvas.FillRect(0, 0, prevwidth, Height, 0);
  for i := 0 to High(Elements) do
    Elements[i].Draw(MemCanvas);

  // set alpha
  if AlphaBits <> nil then
  begin
    c := MemCanvas.Bits;
    ec := @PChar(c)[prevwidth*Height*4];
    a := AlphaBits;
    while PChar(c) < PChar(ec) do
    begin
      c^ := c^ or (a^ shl 24);
      {
      // premultiplied
      PChar(c)[0] := Char(Byte(PChar(c)[0]) * a^ div 255);
      PChar(c)[1] := Char(Byte(PChar(c)[1]) * a^ div 255);
      PChar(c)[2] := Char(Byte(PChar(c)[2]) * a^ div 255);
      }
      Inc(c);
      Inc(a);
    end;
  end;

  pt.x := 0;
  pt.y := 0;
  size.cx := prevwidth;
  size.cy := Height;
  bf.BlendOp := AC_SRC_OVER;
  bf.BlendFlags := 0;
  bf.SourceConstantAlpha := 255;
  bf.AlphaFormat := AC_SRC_ALPHA;
  UpdateLayeredWindow(
    Handle, 0, nil, @size, MemCanvas.Handle, @pt, 0, @bf, ULW_ALPHA);

  if (Current.Y <> Height) or (prevwidth <> Width) then
  begin
    Height := Current.Y;
    SetSize(Width, Height);
    MemCanvas.SetSize(Width, Height);
    ReallocMem(AlphaBits, Width*Height*4);
  end;
end;

procedure TNkMonMainForm.MouseDown(const Button: TMouseButton; CX, CY: Longint);
begin
  StartDrag;
end;

procedure TNkMonMainForm.ShowContextMenu(SX, SY: Longint);
begin
  with TPopupMenu.Create do
    try
      Command.AddMenuItems(Handle, MenuItems);
      CheckItem(Command.Id(cmStayOnTop), StayOnTop);
      Popup(Self.Handle, SX, SY);
    finally
      Free;
    end;
end;

procedure TNkMonMainForm.DispatchCommand(Id: Longint);
begin
  Command.DispatchCommand(Id);
end;

{ Command Methods }

procedure TNkMonMainForm.cmReloadLayout;
begin
  LoadLayout;
end;

procedure TNkMonMainForm.cmStayOnTop;
begin
  SetStayOnTop(not StayOnTop);
end;

procedure TNkMonMainForm.cmAbout;
begin
  with TAboutDialog.Create do
    try
      ProfDir := MainForm.ProfDir;
      ShowModal(Self);
    finally
      Free;
    end;
end;

procedure TNkMonMainForm.cmExit;
begin
  Close;
end;

end.
