unit NLib;

interface

uses NTypes;

type
  { Exceptions }

  Exception = class(TObject)
    Msg: string;
  end;

  EError = class(Exception)
    constructor Create(const Msg: string);
  end;

  EOSError = class(Exception)
    ErrNo: Integer;
    Path: WideString;
    constructor Create; overload;
    constructor Create(ErrNo: Integer); overload;
    constructor Create(const Path: WideString); overload;
  end;

  { Types and Classes }

  TCmpInt = function (const A, B: Integer): Integer;
  TCmpStr = function (const A, B: string): Integer;
  TCmpWideStr = function (const A, B: WideString): Integer;

  TSeekWhence = (swSet, swCur, swEnd);

  TStream = class(TObject)
  protected
    ReadP, EndP, Buf: PChar;
  public
    destructor Destroy; override;
    function Read(var Buf; NBytes: Cardinal): Cardinal; virtual; abstract;
    function Write(const Buf; NBytes: Cardinal): Cardinal; virtual; abstract;
    function Seek(Offset: Int64; Whence: TSeekWhence=swSet): Int64; virtual;
        abstract;
    function Seekable: Boolean; virtual; abstract;
    function ReadLine(var S: string): Boolean;
    function ReadLines: TStringArray;
  end;

  TBytesStream = class(TStream)
    Data: PChar;
    DataSize: Cardinal;
    Position: Cardinal;
    HasData: Boolean;
    constructor Create(const InitialBytes; NBytes: Cardinal;
        NoCopy: Boolean=False); overload;
    constructor Create(const S: string; NoCopy: Boolean=False); overload;
    destructor Destroy; override; 
    function Read(var Buf; NBytes: Cardinal): Cardinal; override;
    function ReadNoCopy(var Buf: PChar; NBytes: Cardinal): Cardinal;
    function Write(const Buf; NBytes: Cardinal): Cardinal; override;
    function Seek(Offset: Int64; Whence: TSeekWhence = swSet): Int64; override;
    function Seekable: Boolean; override;
  end;

  TArgParser = object
  private
    ArgInd: Integer;
    FArgs: TWideStringArray;
    Opt: PWideChar;
  public
    function Args: TWideStringArray;
    function GetArg: WideString;
    function GetOpt(var Opt_: WideChar): Boolean;
  end;

function ArgParser(): TArgParser; overload;
function ArgParser(const Args: array of WideString): TArgParser; overload;

function BSwap16(X: Word): Word;
function BSwap32(X: Longword): Longword;
function BSwap64(X: Int64): Int64;

const
  Be16ToH: function (X: Word): Word = BSwap16;
  Be32ToH: function (X: LongWord): LongWord = BSwap32;
  Be64ToH: function (X: Int64): Int64 = BSwap64;
  HToBe16: function (X: Word): Word = BSwap16;
  HToBe32: function (X: LongWord): LongWord = BSwap32;
  HToBe64: function (X: Int64): Int64 = BSwap64;

function DivMod(A, B: Integer; var R: Integer): Integer;

function SplitCommandLine(const S: string): TStringArray;
function SplitCommandLineW(const S: WideString): TWideStringArray;
function JoinCommandLine(const Args: array of WideString): WideString;

function IStr(Value: Longint): string; overload;
function IStr(Value: Int64): string; overload;
function FStr(Value: Double; Digits: Integer): string;
function IHex(Value: Longint): string;
function I64Hex(Value: Int64): string;
function UnitFormat(Value: Int64): string;
function NumFormat(Value: Int64): string;

function SInt(const S: string): Longint;
function SInt64(const S: string): Int64;
function SFloat(const S: string): Double;
function HInt(const S: string): Longint;

function FindBytes(const Buf; NBytes: Integer; const Sub: string;
    Start: Integer=0): Integer;

function SliceArray(const StrArray: array of string; Start: Integer):
    TStringArray; overload;
function SliceArray(const StrArray: array of WideString; Start: Integer):
    TWideStringArray; overload;
function StringArray(const Args: array of string): TStringArray;
function StringArrayW(const Args: array of WideString): TWideStringArray;

function BaseName(const Path: WideString): WideString;
function DirName(const Path: WideString): WideString;

function RoundOff(Value: Double): Longint;

function Index(IntArray: array of Integer; Value: Integer): Integer; overload;
function Index(StrArray: array of string; const Value: String): Integer;
    overload;
function Cmp(const A, B: Integer): Integer; overload;
function Cmp(const A, B: string): Integer; overload;
function Cmp(const A, B: WideString): Integer; overload;
procedure Sort(var IntArray: array of Integer; Cmp: TCmpInt=nil); overload;
procedure Sort(var StrArray: array of string; Cmp: TCmpStr=nil); overload;
procedure Sort(var WideStrArray: array of WideString; Cmp: TCmpWideStr=nil); overload;

procedure Print(const S: string = ''); overload;
procedure Print(const S: string; const Eol: string); overload;
procedure Print(const F: TStream; const S: string = ''); overload;
procedure Print(const F: TStream; const S: string; const Eol: string); overload;
function FormatException(const E: TObject): WideString;
procedure SysFatal;

function Format(const S: string; const Args: array of const): string;

implementation

uses NSys;

function BSwap16(X: Word): Word;
asm
  rol ax, 8
end;

function BSwap32(X: Longword): Longword;
asm
  bswap eax
end;

function BSwap64(X: Int64): Int64;
begin
  Result := Int64(BSwap32(X)) shl 32 or BSwap32(X shr 32);
end;

function DivMod(A, B: Integer; var R: Integer): Integer;
asm
  // R := A mod B;
  // Result := A div B;
  push ebx
  mov ebx, edx
  cdq
  idiv ebx
  mov ebx, R
  mov [ebx], edx
  pop ebx
end;

function SplitCommandLine(const S: string): TStringArray;
var
  i, argc: Integer;
  q, qq: Boolean;
  t: string;
begin
  SetLength(Result, 1024);
  argc := 0;
  q := False;
  qq := False;
  i := 0;
  t := '';
  while i < Length(S) do
  begin
    Inc(i);
    if qq then
    begin
      t := t + S[i];
      qq := False;
      Continue;
    end;
    case S[i] of
      '"':
        if q and (S[i+1] = '"') then
        begin
          qq := True;
          Continue;
        end else
          q := not q;
      ' ', #9, #13, #10:
        if q then
          t := t + S[i]
        else if t <> '' then
        begin
          Result[argc] := t;
          if argc = High(Result) then
            SetLength(Result, Length(Result) + 1024);
          Inc(argc);
          t := '';
        end;
      else
        t := t + S[i]
    end;
  end;
  if t <> '' then
  begin
    Result[argc] := t;
    Inc(argc);
  end;
  SetLength(Result, argc);
end;

function SplitCommandLineW(const S: WideString): TWideStringArray;
var
  i, argc: Integer;
  q, qq: Boolean;
  t: WideString;
begin
  SetLength(Result, 1024);
  argc := 0;
  q := False;
  qq := False;
  i := 0;
  t := '';
  while i < Length(S) do
  begin
    Inc(i);
    if qq then
    begin
      t := t + S[i];
      qq := False;
      Continue;
    end;
    case S[i] of
      '"':
        if q and (S[i+1] = '"') then
        begin
          qq := True;
          Continue;
        end else
          q := not q;
      ' ', #9, #13, #10:
        if q then
          t := t + S[i]
        else if t <> '' then
        begin
          Result[argc] := t;
          if argc = High(Result) then
            SetLength(Result, Length(Result) + 1024);
          Inc(argc);
          t := '';
        end;
      else
        t := t + S[i]
    end;
  end;
  if t <> '' then
  begin
    Result[argc] := t;
    Inc(argc);
  end;
  SetLength(Result, argc);
end;

function JoinCommandLine(const Args: array of WideString): WideString;
  function quote(const S: WideString; QuoteCount: Integer): WideString;
  var
    i, j: Integer;
  begin
    SetString(Result, nil, Length(S) + QuoteCount);
    j := 1;
    for i := 1 to Length(S) do
    begin
      if S[i] = '"' then
      begin
        Result[j] := '"';
        Inc(j);
      end;
      Result[j] := S[i];
      Inc(j);
    end;
  end;
var
  i, j, len: Integer;
  space: Boolean;
  quotecount: Integer;
  quoted: TWideStringArray;
  p: PWideChar;
begin
  len := 0;
  SetLength(quoted, Length(Args));
  for i := 0 to High(Args) do
  begin
    space := False;
    quotecount := 0;
    for j := 1 to Length(Args[i]) do
    begin
      case Args[i][j] of
        #9, ' ':
          if (quotecount and 1) = 0 then
            space := True;
        '"': Inc(quotecount);
      end;
    end;
    if space or ((quotecount and 1) = 1) then
      quoted[i] := '"' + quote(Args[i], quotecount) + '"'
    else
      quoted[i] := Args[i];
    Inc(len, Length(quoted[i]));
  end;

  if len = 0 then
  begin
    Result := '';
    Exit;
  end;

  SetLength(Result, len + High(Args));
  p := PWideChar(Result);
  i := 0;
  while True do
  begin
    Move(quoted[i][1], p^, Length(quoted[i]) * SizeOf(WideChar));
    Inc(p, Length(quoted[i]));
    if i = High(quoted) then
      Break;
    p^ := ' ';
    Inc(p);
    Inc(i);
  end;
end;

function IStr(Value: Longint): string;
var
  buf: array[0..10] of Char;
  p, ep: PChar;
begin
  ep := @PChar(@buf)[SizeOf(buf)];
  {
  p := ep;
  if Value < 0 then
  begin
    sign := True;
    Value := -Value;
  end else
    sign := False;
  repeat
    r := Value mod 10;
    Value := Value div 10;
    Dec(p);
    p^ := Char(r + Ord('0'));
  until Value = 0;
  if sign then
  begin
    Dec(p);
    p^ := '-';
  end;
  }
  asm
    push ebx
    push edi
    mov eax, Value
    mov ebx, eax
    mov edi, ep
    mov ecx, 10
    test eax, eax
    jns @loop
    neg eax

  @loop:
    xor edx, edx
    div ecx
    add edx, '0'
    dec edi
    mov byte ptr [edi], dl
    cmp eax, 0
    jnz @loop

    test ebx, ebx
    jge @exit
    dec edi
    mov byte ptr [edi], '-'

  @exit:
    mov p, edi
    pop edi
    pop ebx
  end;
  SetString(Result, p, ep - p);
end;

function IStr(Value: Int64): string;
var
  sign: Boolean;
  buf: array[0..19] of Char;
  p, ep: PChar;
  r: Integer;
begin
  ep := @PChar(@buf)[SizeOf(buf)];
  p := ep;
  if Value < 0 then
  begin
    sign := True;
    Value := -Value;
  end else
    sign := False;
  repeat
    r := Value mod 10;
    Value := Value div 10;
    Dec(p);
    p^ := Char(r + Ord('0'));
  until Value = 0;
  if sign then
  begin
    Dec(p);
    p^ := '-';
  end;
  SetString(Result, p, ep - p);
end;

function Pow10(N: Integer): Double;
const
  tab: array[0..159] of Double = (
    1e0,  1e1,  1e2,  1e3,  1e4,  1e5,  1e6,  1e7,  1e8,  1e9,
    1e10, 1e11, 1e12, 1e13, 1e14, 1e15, 1e16, 1e17, 1e18, 1e19,
    1e20, 1e21, 1e22, 1e23, 1e24, 1e25, 1e26, 1e27, 1e28, 1e29,
    1e30, 1e31, 1e32, 1e33, 1e34, 1e35, 1e36, 1e37, 1e38, 1e39,
    1e40, 1e41, 1e42, 1e43, 1e44, 1e45, 1e46, 1e47, 1e48, 1e49,
    1e50, 1e51, 1e52, 1e53, 1e54, 1e55, 1e56, 1e57, 1e58, 1e59,
    1e60, 1e61, 1e62, 1e63, 1e64, 1e65, 1e66, 1e67, 1e68, 1e69,
    1e70, 1e71, 1e72, 1e73, 1e74, 1e75, 1e76, 1e77, 1e78, 1e79,
    1e80, 1e81, 1e82, 1e83, 1e84, 1e85, 1e86, 1e87, 1e88, 1e89,
    1e90, 1e91, 1e92, 1e93, 1e94, 1e95, 1e96, 1e97, 1e98, 1e99,
    1e100,1e101,1e102,1e103,1e104,1e105,1e106,1e107,1e108,1e109,
    1e110,1e111,1e112,1e113,1e114,1e115,1e116,1e117,1e118,1e119,
    1e120,1e121,1e122,1e123,1e124,1e125,1e126,1e127,1e128,1e129,
    1e130,1e131,1e132,1e133,1e134,1e135,1e136,1e137,1e138,1e139,
    1e140,1e141,1e142,1e143,1e144,1e145,1e146,1e147,1e148,1e149,
    1e150,1e151,1e152,1e153,1e154,1e155,1e156,1e157,1e158,1e159
  );
var
  m: Integer;
begin
  if N < 0 then
  begin
    N := -N;
    if N < Length(tab) then
      Result := 1 / tab[N]
    else begin
      m := n div 2;
      Result := 1 / (Pow10(m) * Pow10(N-m));
    end;
  end
  else begin
    if N < Length(tab) then
      Result := tab[N]
    else begin
      m := n div 2;
      Result := Pow10(m) * Pow10(N-m);
    end;
  end;
end;

function FStr(Value: Double; Digits: Integer): string;
const
  rlog2 = 0.30102999566398114; // 1/log2(10)
var
  m: Double;
  i, ndig, k, e2, e10, l, len: Integer;
  p0, p1: PInteger;
begin
  p0 := @Value;
  p1 := p0;
  Inc(p1); // little endian

  e2 := p1^ shr 20 and $7ff; // biased exponent
  if e2 = $7ff then
  begin
    if (p0^ = 0) then
    begin
      if (p1^ and $800FFFFF = 0) then
        Result := 'Inf'
      else if (p1^ and $800FFFFF = $80000000) then
        Result := '-Inf'
    end else
      Result := 'NaN';
    Exit;
  end;

  if Value = 0.0 then
  begin
    if Digits = 0 then
      Result := '0'
    else
      Result := '0.' + StringOfChar('0', Digits);
    Exit;
  end;

  Dec(e2, 1022);
  {
    Value = 2^e2 * f
    Value = 10^e10 * m
    e10 = e2 * 1/log2(10)
    m = Value * 1/10^e10
  }
  e10 := Trunc(e2 * rlog2);
  m := Value * Pow10(-e10);

  len := Digits + 3; // +3: '-' and '0.'
  if e10 > 0 then
    Inc(len, e10);
  SetString(Result, nil, len);
  k := 1;
  if m < 0 then
  begin
    m := -m;
    Result[k] := '-';
    Inc(k);
  end;

  if m <> 0 then
  begin
    while m < 1.0 do
    begin
      m := m * 10.0;
      Dec(e10);
    end;
    while m > 10.0 do
    begin
      m := m / 10.0;
      Inc(e10);
    end;
  end;

  m := m + Pow10(-e10-Digits-1); // round off

  ndig := 0;
  if e10 < 0 then
  begin
    Result[k] := '0';
    Inc(k);
    if Digits = 0 then
    begin
      SetLength(Result, k-1);
      Exit;
    end;

    Result[k] := '.';
    Inc(k);

    for i := 1 to -e10-1 do
    begin
      if ndig = Digits then
      begin
        SetLength(Result, k-1);
        Exit;
      end;
      Result[k] := '0';
      Inc(k);
      Inc(ndig);
    end;
  end;

  l := 0;
  while True do
  begin
    i := Trunc(m);
    if not ((i = 0) and (l = 0) and (e10 > 0)) then
    begin
      Result[k] := Char(i+Ord('0'));
      Inc(k);
    end;
    if l > e10 then
    begin
      Inc(ndig);
      if ndig = Digits then
        Break;
    end;
    if l = e10 then
    begin
      if Digits = 0 then
        Break;
      Result[k] := '.';
      Inc(k);
    end;
    Inc(l);
    m := (m - i) * 10;
  end;
  SetLength(Result, k-1);
end;

function IHex(Value: Longint): string;
const
  hexdigs: array[0..15] of Char = '0123456789abcdef';
var
  sign: Boolean;
  i: Integer;
  buf: array[0..9] of Char;
begin
  if Value < 0 then
  begin
    sign := True;
    Value := -Value;
  end else
    sign := False;
  i := SizeOf(buf);
  repeat
    Dec(i);
    buf[i] := hexdigs[Value and $F];
    Value := Value shr 4;
  until Value = 0;
  if sign then
  begin
    Dec(i);
    buf[i] := '-';
  end;
  SetString(Result, PChar(@buf[i]), SizeOf(buf)-i);
end;

function I64Hex(Value: Int64): string;
const
  hexdigs: array[0..15] of Char = '0123456789abcdef';
var
  sign: Boolean;
  i: Integer;
  buf: array[0..17] of Char;
begin
  if Value < 0 then
  begin
    sign := True;
    Value := -Value;
  end else
    sign := False;
  i := SizeOf(buf);
  repeat
    Dec(i);
    buf[i] := hexdigs[Value and $F];
    Value := Value shr 4;
  until Value = 0;
  if sign then
  begin
    Dec(i);
    buf[i] := '-';
  end;
  SetString(Result, PChar(@buf[i]), SizeOf(buf)-i);
end;

// convert to 4 digits or 3 digits + point and unit character
function UnitFormat(Value: Int64): string;
const
  KiroSize = 1024;
  MegaSize = 1024 * 1024;
  GigaSize = 1024 * 1024 * 1024;
var
  u: Char;
  d: Integer;
  n: Double;
  sign: Boolean;
begin
  if Value < 0 then
  begin
    sign := True;
    Value := -Value;
  end else
    sign := False;
  if Value < KiroSize then
  begin
    if sign then
      Value := -Value;
    Result := IStr(Value)
  end
  else begin
    if Value >= MegaSize * 1000 then
    begin
      n := Value / GigaSize;
      u := 'G';
    end
    else if Value >= KiroSize * 1000 then
    begin
      n := Value / MegaSize;
      u := 'M';
    end
    else begin
      n := Value / KiroSize;
      u := 'K';
    end;

    // truncate n
    if n >= 100 then
    begin
      d := 0;
      n := Int(n);
    end
    else if n >= 10 then
    begin
      d := 1;
      n := Int(n * 10) / 10;
    end
    else begin
      d := 2;
      n := Int(n * 100) / 100;
    end;
    if sign then
      n := -n;
    Result := FStr(n, d) + u;
  end;
end;

function NumFormat(Value: Int64): string;
const
  n: Integer = 3;
var
  s: string;
  i, j, len, ofs: Integer;
begin
  s := IStr(Value);
  len := Length(s);
  SetLength(Result, len + (len-1) div n);
  ofs := n - len mod n;
  i := 1;
  for j := 1 to len do
  begin
    Result[i] := s[j];
    Inc(i);
    if (j + ofs) mod 3 = 0 then
    begin
      Result[i] := ',';
      Inc(i);
    end;
  end;
end;

function SInt(const S: string): Longint;
var
  sign: Boolean;
  i: Integer;
begin
  sign := False;
  i := 1;
  while (i <= Length(S)) and (S[i] in [#9, ' ']) do
    Inc(i);
  if i <= Length(S) then
    if S[i] = '+' then
      Inc(i)
    else if S[i] = '-' then
    begin
      sign := True;
      Inc(i);
    end;
  Result := 0;
  while (i <= Length(S)) and (S[i] in ['0'..'9']) do
  begin
    Result := Result * 10 + Ord(S[i]) - Ord('0');
    Inc(i);
  end;
  if sign then
    Result := -Result;
end;

function SInt64(const S: string): Int64;
var
  sign: Boolean;
  i: Integer;
begin
  sign := False;
  i := 1;
  while (i <= Length(S)) and (S[i] in [#9, ' ']) do
     Inc(i);
  if (i <= Length(S)) and (S[i] = '-') then
    if S[i] = '+' then
      Inc(i)
    else if S[i] = '-' then
    begin
      sign := True;
      Inc(i);
    end;
  Result := 0;
  while (i <= Length(S)) and (S[i] in ['0'..'9']) do
  begin
    Result := Result * 10 + Ord(S[i]) - Ord('0');
    Inc(i);
  end;
  if sign then
    Result := -Result;
end;

// TODO: do not use Val()
function SFloat(const S: string): Double;
var
  code: Integer;
begin
  Val(S, Result, code);
end;

function HInt(const S: string): Longint;
var
  sign: Boolean;
  i: Integer;
begin
  sign := False;
  i := 1;
  while (i <= Length(S)) and (S[i] in [#9, ' ']) do
    Inc(i);
  if i <= Length(S) then
    if S[i] = '+' then
      Inc(i)
    else if S[i] = '-' then
    begin
      sign := True;
      Inc(i);
    end;
  Result := 0;
  while i <= Length(S) do
  begin
    case S[i] of
      '0'..'9': Result := Result * 16 + Ord(S[i]) - Ord('0');
      'A'..'F': Result := Result * 16 + Ord(S[i]) - 55;
      'a'..'f': Result := Result * 16 + Ord(S[i]) - 87;
    else
      Break;
    end;
    Inc(i);
  end;
  if sign then
    Result := -Result;
end;


function FindBytes(const Buf; NBytes: Integer; const Sub: string;
    Start: Integer=0): Integer;
var
  i, j, endi, sublen: Integer;
  p: PChar;
begin
  Result := -1;
  sublen := Length(Sub);
  if sublen = 0 then
    Exit;
  endi := NBytes - sublen + 1;
  if endi < Start then
    Exit;
  p := @Buf;
  for i := Start to endi do
    if p[i] = Sub[1] then
    begin
      j := 1;
      while (j < sublen) and (p[i+j] = Sub[1+j]) do 
        Inc(j);
      if j = sublen then
      begin
        Result := i;
        Exit;
      end;
    end;
end;

function SliceArray(const StrArray: array of string; Start: Integer):
    TStringArray;
var
  i, len: Integer;
begin
  len := High(StrArray) - Start + 1;
  if len < 0 then
    len := 0;
  SetLength(Result, len);
  for i := 0 to High(Result) do
    Result[i] := StrArray[Start+i];
end;

function SliceArray(const StrArray: array of WideString; Start: Integer):
    TWideStringArray;
var
  i, len: Integer;
begin
  len := High(StrArray) - Start + 1;
  if len < 0 then
    len := 0;
  SetLength(Result, len);
  for i := 0 to High(Result) do
    Result[i] := StrArray[Start+i];
end;

function StringArray(const Args: array of string): TStringArray;
var
  i: Integer;
begin
  SetLength(Result, Length(Args));
  for i := 0 to High(Args) do
    Result[i] := Args[i];
end;

function StringArrayW(const Args: array of WideString): TWideStringArray;
var
  i: Integer;
begin
  SetLength(Result, Length(Args));
  for i := 0 to High(Args) do
    Result[i] := Args[i];
end;

function BaseName(const Path: WideString): WideString;
var
  s: WideString;
begin
  SplitPath(Path, s, Result);
end;

function DirName(const Path: WideString): WideString;
var
  s: WideString;
begin
  SplitPath(Path, Result, s);
end;

function RoundOff(Value: Double): Longint;
begin
  if Value >= 0 then
    Result := Trunc(Value + 0.5)
  else
    Result := Trunc(Value - 0.5);
end;

function Index(IntArray: array of Integer; Value: Integer): Integer;
var
  i: Integer;
begin
  for i := Low(IntArray) to High(IntArray) do
    if IntArray[i] = Value then
    begin
      Result := i;
      Exit;
    end;
  Result := -1;
end;

function Index(StrArray: array of string; const Value: String): Integer;
var
  i: Integer;
begin
  for i := Low(StrArray) to High(StrArray) do
    if StrArray[i] = Value then
    begin
      Result := i;
      Exit;
    end;
  Result := -1;
end;

function Cmp(const A, B: Integer): Integer;
begin
  if A > B then
    Result := 1
  else if A < B then
    Result := -1
  else
    Result := 0;
end;

function Cmp(const A, B: string): Integer;
begin
  if A > B then
    Result := 1
  else if A < B then
    Result := -1
  else
    Result := 0;
end;

function Cmp(const A, B: WideString): Integer;
begin
  if A > B then
    Result := 1
  else if A < B then
    Result := -1
  else
    Result := 0;
end;

procedure Sort(var IntArray: array of Integer; Cmp: TCmpInt=nil);
  procedure merge(var Src, Dest: array of Integer; L, H: Integer);
  var
    i, j, k, m: Integer;
  begin
    if L >= H then
      Exit;
    m := (H + L) div 2;
    merge(Dest, Src, L, m);
    merge(Dest, Src, m+1, H);
    j := L;
    k := m+1;
    for i := L to H do
    begin
      if (k > H) or (j <= m) and (Cmp(Src[j], Src[k]) <= 0) then
      begin
        Dest[i] := Src[j];
        Inc(j);
      end
      else begin 
        Dest[i] := Src[k];
        Inc(k);
      end;
    end;
  end;
var
  i: Integer;
  aux: array of Integer;
begin
  if not Assigned(Cmp) then
    Cmp := NLib.Cmp;
  SetLength(aux, Length(IntArray));
  for i := 0 to High(aux) do
    aux[i] := IntArray[i];
  merge(aux, IntArray, 0, High(aux));
end;

procedure Sort(var StrArray: array of string; Cmp: TCmpStr=nil);
  procedure merge(var Src, Dest: array of string; L, H: Integer);
  var
    i, j, k, m: Integer;
  begin
    if L >= H then
      Exit;
    m := (H + L) div 2;
    merge(Dest, Src, L, m);
    merge(Dest, Src, m+1, H);
    j := L;
    k := m+1;
    for i := L to H do
    begin
      if (k > H) or (j <= m) and (Cmp(Src[j], Src[k]) <= 0) then
      begin
        Dest[i] := Src[j];
        Inc(j);
      end
      else begin 
        Dest[i] := Src[k];
        Inc(k);
      end;
    end;
  end;
var
  i: Integer;
  aux: array of string;
begin
  if not Assigned(Cmp) then
    Cmp := NLib.Cmp;
  SetLength(aux, Length(StrArray));
  for i := 0 to High(aux) do
    aux[i] := StrArray[i];
  merge(aux, StrArray, 0, High(aux));
end;

procedure Sort(var WideStrArray: array of WideString; Cmp: TCmpWideStr=nil);
  procedure merge(var Src, Dest: array of WideString; L, H: Integer);
  var
    i, j, k, m: Integer;
  begin
    if L >= H then
      Exit;
    m := (H + L) div 2;
    merge(Dest, Src, L, m);
    merge(Dest, Src, m+1, H);
    j := L;
    k := m+1;
    for i := L to H do
    begin
      if (k > H) or (j <= m) and (Cmp(Src[j], Src[k]) <= 0) then
      begin
        Dest[i] := Src[j];
        Inc(j);
      end
      else begin 
        Dest[i] := Src[k];
        Inc(k);
      end;
    end;
  end;
var
  i: Integer;
  aux: array of WideString;
begin
  if not Assigned(Cmp) then
    Cmp := NLib.Cmp;
  SetLength(aux, Length(WideStrArray));
  for i := 0 to High(aux) do
    aux[i] := WideStrArray[i];
  merge(aux, WideStrArray, 0, High(aux));
end;

procedure Print(const S: string = '');
var
  t: string;
begin
  t := S + EOL;
  StdOut.Write(t[1], Length(t));
end;

procedure Print(const S: string; const Eol: string);
var
  t: string;
begin
  t := S + Eol;
  StdOut.Write(t[1], Length(t));
end;

procedure Print(const F: TStream; const S: string = '');
var
  t: string;
begin
  t := S + EOL;
  F.Write(t[1], Length(t));
end;

procedure Print(const F: TStream; const S: string; const Eol: string);
var
  t: string;
begin
  t := S + Eol;
  F.Write(t[1], Length(t));
end;

function FormatException(const E: TObject): WideString;
begin
  Result := ProgName + ': ';
  if E is Exception then
  begin
    if E is EOSError then
      with E as EOSError do
        if Path <> '' then
          Result := Result + Path + ': ';
    with E as Exception do
      Result := Result + Msg
  end else
    Result := Result + 'unknown error';
end;

procedure SysFatal;
begin
  Print(StdErr, FormatException(ExceptObject));
  Halt(1);
end;

function Format(const S: string; const Args: array of const): string;
  function ToStr(const X: TVarRec; Typc: Char = 'd'): string;
  begin
    with X do
      case VType of
        vtAnsiString: Result := string(VAnsiString);
        vtChar: Result := VChar;
        vtInteger:
          if Typc = 'x' then
            Result := IHex(VInteger)
          else
            Result := IStr(VInteger);
        vtInt64: Result := IStr(VInt64^);
        vtString: Result := VString^;
        vtWideString: Result := PWideChar(VWideString);
      end;
  end;
type
  TState = (sIdx, sLit, sWid, sWidx, sTerm);
var
  i: Integer;
  state: TState;
  t, widstr, idxstr, padstr, widxstr: string;
  c, padc, typc, alignc: Char;
begin
  i := 0;
  state := sLit;
  typc := 'd';
  widstr := '';
  widxstr := '';
  idxstr := '';
  alignc := '<';
  Result := '';
  while i < Length(S) do
  begin
    Inc(i);
    c := S[i];
    case state of
      sLit:
        case c of
          '{':
            if S[i+1] = '{' then
            begin
              Result := Result + c;
              Inc(i);
            end else
              state := sIdx;
          '}':
            if S[i+1] = '}' then
            begin
              Result := Result + c;
              Inc(i);
            end;
        else
          Result := Result + c;
        end;
      sIdx:
        case c of
          '}': state := sTerm;
          ':': state := sWid;
          '0'..'9': idxstr := idxstr + c;
        end;
      sWid:
        case c of
          '{': state := sWidx;
          '}': state := sTerm;
          '0'..'9': widstr := widstr + c;
          '<', '>': alignc := c;
          'd', 'x': typc := c;
        end;
      sWidx:
        case c of
          '}': 
            begin
              widstr := widstr + ToStr(args[SInt(widxstr)]);
              widxstr := '';
              state := sWid;
            end;
          '0'..'9': widxstr := widxstr + c;
        end;
    end;
    if state = sTerm then
    begin
      t := ToStr(args[SInt(idxstr)], typc);
      if (Length(widstr) > 0) and (widstr[1] = '0') then
      begin
        alignc := '>';
        padc := '0'
      end else
        padc := ' ';
      padstr := StringOfChar(padc, SInt(widstr)-Length(t));
      if alignc = '<' then
        Result := Result + t + padstr
      else
        Result := Result + padstr + t;
      idxstr := '';
      widstr := '';
      typc := 'd';
      alignc := '<';
      state := sLit;
    end;
  end;
end;


{ Exceptions }

constructor EError.Create(const Msg: string);
begin
  Self.Msg := Msg;
end;

constructor EOSError.Create;
begin
  ErrNo := SysErrNo;
  Msg := SysError(ErrNo);
end;

constructor EOSError.Create(ErrNo: Integer);
begin
  Self.ErrNo := ErrNo;
  Msg := SysError(ErrNo);
end;

constructor EOSError.Create(const Path: WideString);
begin
  ErrNo := SysErrNo;
  Msg := SysError(ErrNo);
  Self.Path := Path;
end;


{ TStream }

destructor TStream.Destroy;
begin
  if Buf <> nil then
    FreeMem(Buf);
  inherited;
end;

function TStream.ReadLine(var S: string): Boolean;
const
  bufsize = 8192;
var
  len, dlen: Integer;
  sp: PChar;
  foundlf: Boolean;
begin
  if Buf = nil then
    GetMem(Buf, bufsize);

  dlen := 0;
  foundlf := False;
  while not foundlf do
  begin
    if ReadP = EndP then
    begin
      len := Read(Buf^, bufsize);
      if len = 0 then
      begin
        Result := dlen <> 0;
        Exit;
      end;
      ReadP := Buf;
      EndP := Buf + len;
    end;

    sp := ReadP;
    while ReadP < EndP do
    begin
      if ReadP^ = #10 then
      begin
        foundlf := True;
        Inc(ReadP);
        Break;
      end;
      Inc(ReadP);
    end;

    SetLength(S, dlen + (ReadP - sp));
    Move(sp^, S[dlen+1], ReadP - sp);
    Inc(dlen, ReadP - sp);
  end;
  Result := True;
end;

function TStream.ReadLines: TStringArray;
var
  i: Integer;
  s: string;
begin
  SetLength(Result, 1024);
  i := 0;
  while ReadLine(s) do
  begin
    if i = Length(Result) then
      SetLength(Result, i + i div 4);
    Result[i] := s;
    Inc(i);
  end;
  SetLength(Result, i);
end;

{ TBytesStream }

constructor TBytesStream.Create(const InitialBytes; NBytes: Cardinal;
    NoCopy: Boolean=False);
begin
  DataSize := NBytes;
  if NoCopy then
    Data := @InitialBytes
  else begin
    // TODO: raise exception if out of memory
    GetMem(Data, DataSize);
    HasData := True;
    Move(InitialBytes, Data^, DataSize);
  end;
  Seek(0);
end;

constructor TBytesStream.Create(const S: string; NoCopy: Boolean=False);
begin
  Create(S[1], Length(S), NoCopy);
end;

destructor TBytesStream.Destroy;
begin
  if HasData then
    FreeMem(Data);
end;

function TBytesStream.Read(var Buf; NBytes: Cardinal): Cardinal;
begin
  if NBytes > (DataSize - Position) then
    NBytes := DataSize - Position;
  Move(Data[Position], Buf, NBytes);
  Inc(Position, NBytes);
  Result := NBytes;
end;

function TBytesStream.ReadNoCopy(var Buf: PChar; NBytes: Cardinal): Cardinal;
begin
  if NBytes > (DataSize - Position) then
    NBytes := DataSize - Position;
  Buf := @Data[Position];
  Inc(Position, NBytes);
  Result := NBytes;
end;

function TBytesStream.Write(const Buf; NBytes: Cardinal): Cardinal;
begin
  if NBytes > (DataSize - Position) then
    NBytes := DataSize - Position;
  Move(Buf, Data[Position], NBytes);
  Inc(Position, NBytes);
  Result := NBytes;
end;

// TODO: support swCur and swEnd
function TBytesStream.Seek(Offset: Int64; Whence: TSeekWhence = swSet): Int64;
begin
  Position := Offset;
  if Position > DataSize then
    Position := DataSize;
  Result := Position;
end;

function TBytesStream.Seekable: Boolean;
begin
  Result := True;
end;

{ TArgParser }

function ArgParser(): TArgParser;
begin
  Result.FArgs := Argv;
  Result.ArgInd := 1;
end;

function ArgParser(const Args: array of WideString): TArgParser;
var
  i: Integer;
begin
  SetLength(Result.FArgs, Length(Args));
  for i := 0 to High(Result.FArgs) do 
    Result.FArgs[i] := Args[i];
end;

function TArgParser.Args: TWideStringArray;
begin
  Result := SliceArray(FArgs, ArgInd);
end;

function TArgParser.GetOpt(var Opt_: WideChar): Boolean;
begin
  if Opt <> nil then
    Inc(Opt);

  if (Opt = nil) or (Opt^ = #0) then
  begin
    Result := False;
    if ArgInd = Length(FArgs) then
      Exit;
    Opt := PWideChar(FArgs[ArgInd]);
    if Opt = '--' then
    begin
      Inc(ArgInd);
      Exit;
    end;
    if (Opt^ = '-') and (Length(Opt) > 1) then
      Inc(Opt)
    else
      Exit;
    Inc(ArgInd);
  end;
  Opt_ := Opt^;
  Result := Opt_ <> #0;
end;

function TArgParser.GetArg: WideString;
begin
  Inc(Opt);
  if Opt^ = #0 then
  begin
    if ArgInd < Length(FArgs) then
    begin
      Result := FArgs[ArgInd];
      Inc(ArgInd); 
    end else
      Result := '';
  end else
    Result := Opt;
  Opt := nil;
end;

end.
