unit NNet;

interface

uses WinSock;

type
  TEvent = procedure (const Param) of object;

  TSock = class(TObject)
    OnClose: TEvent;
    OnConnect: TEvent;
    OnError: TEvent;
    OnFail: TEvent;
    OnRecv: TEvent;
    Soc: TSocket;
    procedure CreateSocket(Type_: Integer);
    destructor Destroy; override;
  end;

  TUdp = class(TSock)
    constructor Create;
    procedure Bind(LocalPort: Integer);
    function Send(const Buf; NBytes: Integer; const IpAddr: string;
      Port: Integer): Integer;
    function Recv(var Buf; NBytes: Integer; var IpAddr: string): Integer;
  end;

procedure GetAddress(const HostName: string; Done: TEvent; Fail: TEvent);
function GetHostName: string;

implementation

uses Windows, Messages, NLib;

type
  TGetAddressData = record
    HostEnt: array[0..MAXGETHOSTSTRUCT-1] of Char;
    Done: TEvent;
    Fail: TEvent;
  end;
  PGetAddressData = ^TGetAddressData;

const
  WM_APP_GETADDRESS = WM_APP;
  WM_APP_ASYNCSELECT = WM_APP+1;

var
  NetWnd: HWND;
  WSAData: TWSAData;

function AddrStr(const Address): string;
begin
  Result := IStr(Byte(PChar(@Address)[0]))+'.'+
            IStr(Byte(PChar(@Address)[1]))+'.'+
            IStr(Byte(PChar(@Address)[2]))+'.'+
            IStr(Byte(PChar(@Address)[3]));
end;

function SockPropName(Soc: TSocket): string;
begin
  Result := 'S' + IStr(Soc);
end;

function WindowProc(hWnd: HWND; Msg: UINT; wParam: WPARAM; lParam: LPARAM):
  LRESULT; stdcall;
var
  getaddressdata: PGetAddressData;
  error: Integer;
  hostname: string;
  propname: string;
  sock: TSock;
begin
  Result := 0;
  case Msg of
    WM_APP_GETADDRESS:
      begin
        propname := IStr(wParam);
        getaddressdata := PGetAddressData(GetProp(hWnd, PChar(propname)));
        if getaddressdata = nil then
          Exit;
        try
          error := WSAGetAsyncError(lParam);
          if error <> 0 then
          begin
            getaddressdata.Fail(error);
            Exit;
          end;
          hostname := AddrStr(
              PInAddr(PHostEnt(@getaddressdata.HostEnt).h_addr_list^)^);
          getaddressdata.Done(hostname);
        finally
          Dispose(getaddressdata);
          RemoveProp(hWnd, PChar(propname));
        end;
      end;

    WM_APP_ASYNCSELECT:
      begin
        propname := SockPropName(wParam);
        sock := TSock(GetProp(hWnd, PChar(propname)));
        if sock = nil then
          Exit;

        error := WSAGetSelectError(lParam);
        if error <> 0 then
        begin
          if Assigned(sock.OnError) then
            sock.OnError(error);
          Exit;
        end;

        case WSAGetSelectEvent(lParam) of
          FD_READ:
            if Assigned(sock.OnRecv) then
              sock.OnRecv(sock);
          FD_CONNECT:
            if Assigned(sock.OnConnect) then
              sock.OnConnect(sock);
          FD_CLOSE:
            begin
              if Assigned(sock.OnRecv) then
                sock.OnRecv(sock);
              if Assigned(sock.OnClose) then
                sock.OnClose(sock);
            end;
        end;
      end;
  else
    Result := DefWindowProc(hWnd, Msg, wParam, lParam);
  end;
end;

function MakeWindow: HWND;
const
  ClassName = 'NNetWindowClass';
var
  wc: TWndClass;
begin
  FillChar(wc, SizeOf(wc), 0);
  wc.lpfnWndProc := @WindowProc;
  wc.hInstance := HInstance;
  wc.lpszClassName := ClassName;
  if RegisterClass(wc) = 0 then
    raise EOSError.Create;
  Result := CreateWindowEx(WS_EX_TOOLWINDOW, ClassName, nil, WS_POPUP, 0, 0,
    0, 0, 0, 0, HInstance, nil);
  if Result = 0 then
    raise EOSError.Create;
end;

procedure GetAddress(const HostName: string; Done: TEvent; Fail: TEvent);
var
  h: THandle;
  data: PGetAddressData;
begin
  if inet_addr(PChar(HostName)) <> INADDR_NONE then
  begin
    Done(HostName);
    Exit;
  end;

  New(data);
  h := WSAAsyncGetHostByName(NetWnd, WM_APP_GETADDRESS, PChar(HostName),
    data.HostEnt, SizeOf(data.HostEnt));
  if h = 0 then
  begin
    Dispose(data);
    Fail(WSAGetLastError);
  end;
  data.Done := Done;
  data.Fail := Fail;
  SetProp(NetWnd, PChar(IStr(h)), DWORD(data));
end;

function GetHostName: string;
var
  buf: Array[0..255] of Char;
begin
  FillChar(buf, SizeOf(buf), 0);
  WinSock.GetHostName(buf, SizeOf(buf));
  Result := buf;
end;

procedure TSock.CreateSocket(Type_: Integer);
begin
  Soc := socket(AF_INET, Type_, 0);
  if Soc = INVALID_SOCKET then
    raise EOSError.Create;
  if WSAAsyncSelect(Soc, NetWnd, WM_APP_ASYNCSELECT,
      FD_READ or FD_CONNECT or FD_CLOSE) = SOCKET_ERROR then
  begin
    closesocket(Soc);
    raise EOSError.Create;
  end;
  SetProp(NetWnd, PChar(SockPropName(Soc)), DWORD(Self));
end;

destructor TSock.Destroy;
begin
  RemoveProp(NetWnd, PChar(SockPropName(Soc)));
  closesocket(Soc);
end;


{ TUdp }

constructor TUdp.Create;
begin
  CreateSocket(SOCK_DGRAM);
end;

procedure TUdp.Bind(LocalPort: Integer);
var
  sockaddr: TSockAddr;
begin
  FillChar(sockaddr, SizeOf(sockaddr), 0);
  sockaddr.sin_family := AF_INET;
  sockaddr.sin_port := htons(LocalPort);
  sockaddr.sin_addr.S_addr := 0;
  if WinSock.bind(Soc, sockaddr, SizeOf(sockaddr)) = SOCKET_ERROR then
    raise EOSError.Create;
end;

function TUdp.Send(const Buf; NBytes: Integer; const IpAddr: string;
  Port: Integer): Integer;
var
  destaddr: TSockAddr;
begin
  FillChar(destaddr, SizeOf(destaddr), 0);
  destaddr.sin_family := AF_INET;
  destaddr.sin_port := htons(Port);
  destaddr.sin_addr.s_addr := inet_addr(PChar(IpAddr));
  Result := sendto(Soc, Pointer(@Buf)^, NBytes, 0, destaddr, SizeOf(destaddr));
  if Result = SOCKET_ERROR then
    raise EOSError.Create;
end;

function TUdp.Recv(var Buf; NBytes: Integer; var IpAddr: string): Integer;
var
  srcaddr: TSockAddr;
  addrlen: Integer;
begin
  addrlen := SizeOf(srcaddr);
  Result := recvfrom(Soc, Buf, NBytes, 0, srcaddr, addrlen);
  if Result = SOCKET_ERROR then
    raise EOSError.Create;
  IpAddr := AddrStr(srcaddr.sin_addr.s_addr);
end;

initialization
  WSAStartUp($0101, WSAData);
  NetWnd := MakeWindow;

finalization
  if NetWnd <> 0 then
    DestroyWindow(NetWnd);
  WSACleanUp;

end.
